<?php

namespace Drupal\citeref_field\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\MenuLinkManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure citeref field settings for this site.
 *
 * The config form builds checking the status of external APIs
 * for the citation or reference field.
 */
class CiterefFieldSettingsForm extends ConfigFormBase {
  /**
   * An http client.
   */
  protected ClientInterface $httpClient;

  /**
   * An messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The cache menu instance.
   */
  protected CacheBackendInterface $cacheMenu;

  /**
   * The menu link manager instance.
   */
  protected MenuLinkManagerInterface $menuLinkManager;

  /**
   * The config factory for the form.
   */
  public function __construct(
        ConfigFactoryInterface $config_factory,
        MenuLinkManagerInterface $menuLinkManager,
        CacheBackendInterface $cacheMenu,
        ClientInterface $http_client,
        MessengerInterface $messenger
    ) {
    parent::__construct($config_factory);
    $this->cacheMenu = $cacheMenu;
    $this->menuLinkManager = $menuLinkManager;
    $this->httpClient = $http_client;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('config.factory'),
          $container->get('plugin.manager.menu.link'),
          $container->get('cache.menu'),
          $container->get('http_client'),
          $container->get('messenger')
      );
  }

  /**
   * Set up the form ID.
   */
  public function getFormId() {
    return 'citeref_field_settings';
  }

  /**
   * Get the config file name.
   */
  protected function getEditableConfigNames() {
    return ['citeref_field.settings'];
  }

  /**
   * Build the form for API checking routine.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('citeref_field.settings');
    $form['doi_inline_form'] = [
      '#type' => 'container',
    ];

    $form['doi_inline_form']['doi_url'] = [
      '#title' => $this->t('DOI API'),
      '#type' => 'textfield',
      '#size' => 80,
      '#default_value' => $config->get('doi_url'),
      '#placeholder' => $this->t('DOI API'),
      '#attributes' => [
        'style' => 'width: 25em;',
      ],
    ];

    $form['doi_inline_form']['check_doi_api_status'] = [
      '#type' => 'submit',
      '#value' => $this->t('DOI API Status'),
      '#submit' => [[$this, 'checkDoiApiStatus']],
      '#limit_validation_errors' => [],
    ];

    $form['handle_inline_form'] = [
      '#type' => 'container',
    ];

    $form['handle_inline_form']['handle_net_url'] = [
      '#title' => $this->t('Handle.Net API'),
      '#type' => 'textfield',
      '#size' => 80,
      '#default_value' => $config->get('handle_net_url'),
      '#placeholder' => $this->t('Handle.Net API'),
      '#attributes' => [
        'style' => 'width: 25em;',
      ],
    ];

    $form['handle_inline_form']['check_handle_api_status'] = [
      '#type' => 'submit',
      '#value' => $this->t('Handle API Status'),
      '#submit' => [[$this, 'checkHandleApiStatus']],
      '#limit_validation_errors' => [],
    ];

    $form['ark_inline_form'] = [
      '#type' => 'container',
    ];

    $form['ark_inline_form']['ark_url'] = [
      '#title' => $this->t('ARK API'),
      '#type' => 'textfield',
      '#size' => 80,
      '#default_value' => $config->get('ark_url'),
      '#placeholder' => $this->t('ARK API'),
      '#attributes' => [
        'style' => 'width: 25em;',
      ],
    ];

    $form['ark_inline_form']['check_ark_api_status'] = [
      '#type' => 'submit',
      '#value' => $this->t('ARK API Status'),
      '#submit' => [[$this, 'checkArkApiStatus']],
      '#limit_validation_errors' => [],
    ];

    $form['isbn_inline_form'] = [
      '#type' => 'container',
    ];

    $form['isbn_inline_form']['isbn_url'] = [
      '#title' => $this->t('ISBN API'),
      '#type' => 'textfield',
      '#size' => 80,
      '#default_value' => $config->get('isbn_url'),
      '#placeholder' => $this->t('ISBN API'),
      '#attributes' => [
        'style' => 'width: 25em;',
      ],
    ];

    $form['isbn_inline_form']['check_urn_isbn_api_status'] = [
      '#type' => 'submit',
      '#value' => $this->t('URN ISBN API Status'),
      '#submit' => [[$this, 'checkUrnIsbnApiStatus']],
      '#limit_validation_errors' => [],
    ];

    $form['issn_inline_form'] = [
      '#type' => 'container',
    ];

    $form['issn_inline_form']['issn_url'] = [
      '#title' => $this->t('ISSN API'),
      '#type' => 'textfield',
      '#size' => 80,
      '#default_value' => $config->get('issn_url'),
      '#placeholder' => $this->t('ISSN API'),
      '#attributes' => [
        'style' => 'width: 25em;',
      ],
    ];

    $form['issn_inline_form']['check_urn_issn_api_status'] = [
      '#type' => 'submit',
      '#value' => $this->t('URN ISSN API Status'),
      '#submit' => [[$this, 'checkUrnIssnApiStatus']],
      '#limit_validation_errors' => [],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Validate values for the form elements.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!UrlHelper::isValid(trim($form_state->getValue('doi_url')), TRUE)) {
      $form_state->setErrorByName('doi_url', $this->t('The DOI URL, "@doi_url", is not valid.',
      ['@doi_url' => trim($form_state->getValue('doi_url'))]));
    }
    if (!UrlHelper::isValid(trim($form_state->getValue('handle_net_url')), TRUE)) {
      $form_state->setErrorByName('handle_net_url', $this->t('The Handle.Net URL, "@handle_net_url", is not valid.',
      ['@handle_net_url' => trim($form_state->getValue('handle_net_url'))]));
    }
    if (!UrlHelper::isValid(trim($form_state->getValue('ark_url')), TRUE)) {
      $form_state->setErrorByName('ark_url', $this->t('The ARK URL, "@ark_url", is not valid.',
      ['@ark_url' => trim($form_state->getValue('ark_url'))]));
    }
    if (!UrlHelper::isValid(trim($form_state->getValue('isbn_url')), TRUE)) {
      $form_state->setErrorByName('isbn_url', $this->t('The ISBN URL, "@isbn_url", is not valid.',
      ['@isbn_url' => trim($form_state->getValue('isbn_url'))]));
    }
    if (!UrlHelper::isValid(trim($form_state->getValue('issn_url')), TRUE)) {
      $form_state->setErrorByName('issn_url', $this->t('The ISSN URL, "@issn_url", is not valid.',
      ['@issn_url' => trim($form_state->getValue('issn_url'))]));
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * Accesses the DOI API and displays the message for the search result.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function checkDoiApiStatus(array &$form, FormStateInterface $form_state): void {
    $doi_url = trim($form_state->getUserInput()['doi_url']);
    try {
      $request_url = $doi_url . '/10.1126/science.169.3946.635';
      $request = $this->httpClient->request('GET', $request_url, [
        'headers' => ['Accept' => 'text/x-bibliography; style=apa'],
      ]);

      $status = $request->getStatusCode();
      if ($status != 200) {
        $this->messenger->addError(t("<b>DOI API service is down </b> <br/>
            <b>Website: </b> <a href='@doi_url' target='_blank'>DOI</a> <br/>
            <b>Status Code: </b> @status <br/>",
        [
          '@doi_url' => $doi_url,
          '@status' => $status,
        ]));
      }
      else {
        $result = $request->getBody()->getContents();
        $this->messenger->addMessage(t("
        <div>
          <b>DOI API service is up and running </b><br/>
          <b>Website: </b> <a href='@doi_url' target='_blank'>DOI</a> <br/>
          <b>Endpoint URL: </b>
          <a>@request_url</a>
          <br/>
          <b>Status Code: </b> @status <br/>
          <a href='#' type='button' onclick='document.getElementById(`content`).style.display = (document.getElementById(`content`).style.display == `none`) ? `block` : `none`' class='collapsible'><b>Show/Hide Response Body </b></a> <br/>
          <div id='content' style='display:none'>
            @result <br/>
          </div>
        </div>
        ",
        [
          '@doi_url' => $doi_url,
          '@request_url' => $request_url,
          '@status' => $status,
          '@result' => $result,
        ]));

        $this->config('citeref_field.settings')
          ->set('doi_url', $doi_url)
          ->save();
        $messages = $this->t('The configuration options have been saved.');
        $this->messenger->addStatus($messages);
      }
    }
    catch (RequestException $e) {
      $this->messenger->addError(t("<b>DOI API service is down or not valid. </b> <br/>
            <b>Website: </b> <a href='@doi_url' target='_blank'>DOI</a> <br/>
            <b>Status Code: </b> N/A <br/>",
      ['@doi_url' => $doi_url]));
    }
  }

  /**
   * Accesses the Handle Net API and displays the message for the search result.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function checkHandleApiStatus(array &$form, FormStateInterface $form_state) {
    $handle_net_url = trim($form_state->getUserInput()['handle_net_url']);
    try {
      $request_url = $handle_net_url . '/4263537/4000?type=URL&type=EMAIL';
      $request = $this->httpClient->request('GET', $request_url);
      $status = $request->getStatusCode();
      if ($status != 200) {
        $this->messenger->addError(t("
        <b>Handle API service is down </b> <br/>
        <b>Website: </b> <a href='https://hdl.handle.net' target='_blank'>Handle</a> <br/>
        <b>Status Code: </b> @status <br/>
        "),
        ['@status' => $status]);
      }
      else {
        $json_result = $request->getBody();
        $json_string = json_encode(json_decode($json_result), JSON_PRETTY_PRINT);

        $this->messenger->addMessage(t("
        <div>
          <b>Handle API service is up and running </b><br/>
          <b>Website: </b> <a href='https://hdl.handle.net' target='_blank'>Handle</a> <br/>
          <b>Endpoint URL: </b>
          <a>@request_url</a>
          <br/>
          <b>Status Code: </b> @status <br/>
          <a href='#' type='button' onclick='document.getElementById(`content`).style.display = (document.getElementById(`content`).style.display == `none`) ? `block` : `none`' class='collapsible'><b>Show/Hide Response Body </b></a> <br/>

          <div id='content' style='display:none'>
            <pre>@json_string</pre> <br/>
          </div>
        </div>
        ",
          [
            '@request_url' => $request_url,
            '@status' => $status,
            '@json_string' => $json_string,
          ]));

        $this->config('citeref_field.settings')
          ->set('handle_net_url', $handle_net_url)
          ->save();
        $messages = $this->t('The configuration options have been saved.');
        $this->messenger->addStatus($messages);
      }
    }
    catch (RequestException $e) {
      $this->messenger->addError(t("
        <b>Handle API service is down or not valid. </b> <br/>
        <b>Website: </b> <a href='https://hdl.handle.net' target='_blank'>Handle</a> <br/>
        <b>Status Code: </b> N/A <br/>
        "));
    }
  }

  /**
   * Accesses the N2T API and displays the message for the search result.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function checkArkApiStatus(array &$form, FormStateInterface $form_state): void {
    $ark_url = trim($form_state->getUserInput()['ark_url']);

    try {
      // The ARK resolver.
      $request_url = $ark_url . '67531/metapth346793/??';
      $request = $this->httpClient->request('GET', $request_url);

      $status = $request->getStatusCode();
      if ($status != 200) {
        $this->messenger->addError(t("
        <b>ARK API service is down </b> <br/>
        <b>Website: </b> <a href='https://n2t.net' target='_blank'>ARK</a> <br/>
        <b>Status Code: </b> @status <br/>
        ", ['@status' => $status]));
      }
      else {
        $result = $request->getBody()->getContents();
        $this->messenger->addMessage(t("
        <div>
          <b>ARK API service is up and running </b><br/>
          <b>Website: </b> <a href='https://n2t.net' target='_blank'>ARK</a> <br/>
          <b>Endpoint URL: </b>
          <a>@request_url</a>
          <br/>
          <b>Status Code: </b> @status <br/>
          <a href='#' type='button' onclick='document.getElementById(`content`).style.display = (document.getElementById(`content`).style.display == `none`) ? `block` : `none`' class='collapsible'><b>Show/Hide Response Body </b></a> <br/>

          <div id='content' style='display:none'>
            <pre>@result</pre> <br/>
          </div>
        </div>
        ",
          [
            '@request_url' => $request_url,
            '@status' => $status,
            '@result' => $result,
          ]));

        $this->config('citeref_field.settings')
          ->set('ark_url', $ark_url)
          ->save();
        $messages = $this->t('The configuration options have been saved.');
        $this->messenger->addStatus($messages);
      }
    }
    catch (RequestException $e) {
      $this->messenger->addError(t("
        <b>ARK API service is down or not valid </b> <br/>
        <b>Website: </b> <a href='https://n2t.net' target='_blank'>ARK</a> <br/>
        <b>Status Code: </b> N/A <br/>
        "));
    }
  }

  /**
   * Accesses the URN ISSN API and displays the message for the search result.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function checkUrnIsbnApiStatus(array &$form, FormStateInterface $form_state): void {
    $isbn_url = trim($form_state->getUserInput()['isbn_url']);

    // Google Books API to display basic information about a book.
    try {
      $request_url = $isbn_url . '?q=isbn:978-3-16-148410-0';
      $request = $this->httpClient->request('GET', $request_url);
      $status = $request->getStatusCode();
      if ($status != 200) {
        $this->messenger->addError(t("
        <b>URN ISBN API service is down </b> <br/>
        <b>Website: </b> <a href='@isbn_url' target='_blank'>URN ISBN</a> <br/>
        <b>Status Code: </b> @status <br/>
        ",
          [
            '@$isbn_url' => $isbn_url,
            '@status' => $status,
          ]));
      }
      else {
        $json_result = (string) $request->getBody();
        $json = json_decode($json_result, TRUE);
        $json_string = json_encode($json['items'][0], JSON_PRETTY_PRINT);

        $this->messenger->addMessage(t("
        <div>
          <b>URN ISBN API service is up and running </b><br/>
          <b>Website: </b> <a href='@isbn_url' target='_blank'>URN ISBN</a> <br/>
          <b>Endpoint URL: </b>
          <a>@request_url</a>
          <br/>
          <b>Status Code: </b> @status <br/>
          <a href='#' type='button' onclick='document.getElementById(`content`).style.display = (document.getElementById(`content`).style.display == `none`) ? `block` : `none`' class='collapsible'><b>Show/Hide Response Body </b></a> <br/>

          <div id='content' style='display:none'>
            <pre>@json_string</pre> <br/>
          </div>
        </div>
        ",
          [
            '@isbn_url' => $isbn_url,
            '@request_url' => $request_url,
            '@status' => $status,
            '@json_string' => $json_string,
          ]));

        $this->config('citeref_field.settings')
          ->set('isbn_url', $isbn_url)
          ->save();
        $messages = $this->t('The configuration options have been saved.');
        $this->messenger->addStatus($messages);
      }
    }
    catch (RequestException $e) {
      $this->messenger->addError(t("
        <b>URN ISBN API service is down or not valid. </b> <br/>
        <b>Website: </b> <a href='@isbn_url' target='_blank'>URN ISBN</a> <br/>
        <b>Status Code: </b> N/A <br/>
        ",
        [
          '@$isbn_url' => $isbn_url,
        ]));
    }
  }

  /**
   * Accesses the URN ISBN API and displays the message for the search result.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function checkUrnIssnApiStatus(array &$form, FormStateInterface $form_state): void {
    $issn_url = trim($form_state->getUserInput()['issn_url']);

    try {
      $request_url = $issn_url . '/0028-0836';
      $request = $this->httpClient->request('GET', $request_url);
      $status = $request->getStatusCode();
      if ($status != 200) {
        $this->messenger->addError(t("
        <b>URN ISSN API service is down </b> <br/>
        <b>Website: </b> <a href='https://portal.issn.org' target='_blank'>URN ISSN</a> <br/>
        <b>Status Code: </b> @status <br/>
        ",
          [
            '@status' => $status,
          ]));
      }
      else {
        $this->messenger->addMessage(t("
        <div>
          <b>URN ISSN API service is up and running </b><br/>
          <b>Website: </b> <a href='@issn_url' target='_blank'>URN ISSN</a> <br/>
          <b>Endpoint URL: </b>
          <a>@request_url</a>
          <br/>
          <b>Status Code: </b> @status <br/>
        </div>
        ",
          [
            '@issn_url' => $issn_url,
            '@request_url' => $request_url,
            '@status' => $status,
          ]));

        $this->config('citeref_field.settings')
          ->set('issn_url', $issn_url)
          ->save();
        $messages = $this->t('The configuration options have been saved.');
        $this->messenger->addStatus($messages);
      }
    }
    catch (RequestException $e) {
      $this->messenger->addError(t("
        <b>URN ISSN API service is down or not valid. </b> <br/>
        <b>Website: </b> <a href='https://portal.issn.org' target='_blank'>URN ISSN</a> <br/>
        <b>Status Code: </b> N/A <br/>
        "));
    }
  }

  /**
   * The action function for the form submit.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('citeref_field.settings')
      ->set('doi_url', trim($form_state->getValue('doi_url')))
      ->set('handle_net_url', trim($form_state->getValue('handle_net_url')))
      ->set('ark_url', trim($form_state->getValue('ark_url')))
      ->set('isbn_url', trim($form_state->getValue('isbn_url')))
      ->set('issn_url', trim($form_state->getValue('issn_url')))
      ->save();

    parent::submitForm($form, $form_state);
    $this->cacheMenu->invalidateAll();
    $this->menuLinkManager->rebuild();
  }

}
