<?php

namespace Drupal\citeref_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the configurable formatter.
 *
 * The citation or reference field formatter configures the labels of
 * the citation, link to citation ID, link types, etc.
 * It views the citation information
 * based on the configuration for the citation or reference field.
 *
 * @FieldFormatter(
 *   id = "configurable_citeref_field_formatter",
 *   module = "citeref_field",
 *   label = @Translation("Citation or Reference Field"),
 *   field_types = {
 *     "citeref_field"
 *   }
 * )
 */
class ConfigurableCiterefFieldFormatter extends FormatterBase {
  /*---------------------------------------------------------------------
   *
   * View.
   *
   *---------------------------------------------------------------------*/

  /**
   * Set up values for form elements to view in the theme template.
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    $this->sanitizeSettings();

    /** @var \Drupal\citeref_field\Plugin\Field\FieldType\CiterefFieldItem $item */
    foreach ($items as $delta => $item) {
      $values = $item->getValue();
      $element[$delta] = [
        '#theme' => 'configurable_citeref_field_formatter',
        '#attributes' => [
          'class' => [
            'citeref_field',
          ],
        ],
      ];

      // Citation type Settings.
      $element[$delta]['#citeref_type_label'] = $this->getSetting('citeTypeLabel');

      // Citation style Settings.
      $element[$delta]['#citeref_style_label'] = $this->getSetting('citeStyleLabel');

      // Citation record Settings.
      $element[$delta]['#citeref_record_label'] = $this->getSetting('citeRecordLabel');

      // Citation ID Settings.
      $element[$delta]['#show_id_url'] = $this->getSetting('showIDUrl');
      $element[$delta]['#citeref_id_label'] = $this->getSetting('citeIDLabel');
      $element[$delta]['#id_label_icon_or_link'] = $this->getSetting('citeIDLabelIconOrLink');
      $element[$delta]['#id_open_link_in'] = $this->getSetting('citeIDOpenLinkIn');
      $element[$delta]['#show_id_url_link'] = $this->getSetting('showIDUrlLink');
      $element[$delta]['#id_url_no_referrer'] = $this->getSetting('citeIDurlnoreferrer');
      $element[$delta]['#id_url_no_opener'] = $this->getSetting('citeIDurlnoopener');
      $element[$delta]['#id_url_no_follow'] = $this->getSetting('citeIDurlnofollow');

      // Variables.
      $element[$delta]['#citeref_type'] = $values['citeref_type'];
      $element[$delta]['#citeref_style'] = $values['citeref_style'];
      $element[$delta]['#citeref_id'] = $values['citeref_id'];
      $element[$delta]['#citeref_record'] = $values['citeref_record'];
    }

    return $element;
  }

  /**
   * Set up the default values for form elements in formatter.
   */
  public static function defaultSettings() {
    return array_merge(
          [
            'citeTypeLabel' => 'Citation Type',
            'citeStyleLabel' => 'Citation Style',
            'citeIDLabel' => 'Citation ID',
            'citeRecordLabel' => 'Citation',
            'citeIDLabelIconOrLink' => 'url',
            'showIDUrl' => TRUE,
            'showIDUrlLink' => TRUE,
            'citeIDOpenLinkIn' => '_self',
            'citeIDurlnoreferrer' => FALSE,
            'citeIDurlnoopener' => FALSE,
            'citeIDurlnofollow' => FALSE,
          ],
          parent::defaultSettings());
  }

  /**
   * View current summaries for the citation/reference field formatter.
   */
  public function settingsSummary() {
    // Sanitize current settings.
    $this->sanitizeSettings();

    // Summarize.
    $summary = [];

    if ($this->getSetting('citeTypeLabel')) {
      $summary[] = $this->t('Labels: @citeTypeLabel, @citeStyleLabel, @citeIDLabel, @citeRecordLabel',
      [
        '@citeTypeLabel' => $this->getSetting('citeTypeLabel'),
        '@citeStyleLabel' => $this->getSetting('citeStyleLabel'),
        '@citeIDLabel' => $this->getSetting('citeIDLabel'),
        '@citeRecordLabel' => $this->getSetting('citeRecordLabel'),
      ]);
    }

    if ($this->getSetting('showIDUrlLink')) {
      switch ($this->getSetting('citeIDLabelIconOrLink')) {
        case 'icon':
          $summary[] = $this->t('Citation ID URL: Icon');
          break;

        case 'url':
          $summary[] = $this->t('Citation ID URL: URL');
          break;
      }
    }

    if (!$this->getSetting('showIDUrlLink')) {
      $summary[] = $this->t('Citation ID URL: No link');
    }
    else {
      switch ($this->getSetting('citeIDOpenLinkIn')) {
        case '_self':
          $summary[] = $this->t('Citation ID URL: Open in current tab/window');
          break;

        case '_blank':
          $summary[] = $this->t('Citation ID URL: Open in new tab/window');
          break;
      }

      $text = '';
      if ($this->getSetting('citeIDurlnoreferrer')) {
        $text .= 'noreferrer ';
      }

      if ($this->getSetting('citeIDurlnoopener')) {
        $text .= 'noopener ';
      }

      if ($this->getSetting('citeIDurlnofollow')) {
        $text .= 'nofollow ';
      }

      if (empty($text) === FALSE) {
        $summary[] = 'Citation ID URL: ' . $text;
      }
    }

    return $summary;
  }

  /**
   * Returns an array of choices for how to open a link.
   *
   * @return string[]
   *   Returns an associative array with internal names as keys and
   *   human-readable translated names as values.
   */
  protected static function getOpenLinkInValues() {
    return [
      '_self' => t('Open link in the same tab/window'),
      '_blank' => t('Open link in a new tab/window'),
    ];
  }

  /**
   * Returns an array of list styles.
   *
   * @return string[]
   *   Returns an associative array with internal names as keys and
   *   human-readable translated names as values.
   */
  protected static function getIdLabelOptions() {
    return [
      'icon' => t('Icon next to citation ID'),
      'url' => t('URL'),
    ];
  }

  /**
   * Configure the form elements in the formatter.
   */
  public function settingsForm(array $form, FormStateInterface $formState) {
    $this->sanitizeSettings();

    $weight = 0;
    $marker = rand();
    $elements = [];

    $elements['sectionCitationTypeTitle'] = [
      '#markup' => $this->t('<div><b>Citation Type Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationTypeBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeTypeLabel'] = [
      '#title' => $this->t('Citation Type Label'),
      '#type' => 'textfield',
      '#size' => 10,
      '#default_value' => $this->getSetting('citeTypeLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['sectionCitationStyleTitle'] = [
      '#markup' => $this->t('<div><b>Citation Style Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationStyleBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeStyleLabel'] = [
      '#title' => $this->t('Citation Style Label'),
      '#type' => 'textfield',
      '#size' => 10,
      '#default_value' => $this->getSetting('citeStyleLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['sectionCitationRecordTitle'] = [
      '#markup' => $this->t('<div><b>Citation Text Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationRecordBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeRecordLabel'] = [
      '#title' => $this->t('Citation Text Label'),
      '#type' => 'textfield',
      '#size' => 10,
      '#default_value' => $this->getSetting('citeRecordLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['sectionCitationIDTitle'] = [
      '#markup' => $this->t('<div><b>Citation ID Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationIDBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['showIDUrl'] = [
      '#title' => $this->t('Show Citation ID URL'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('showIDUrl'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
      '#attributes' => [
        'class' => [
          'showIDUrl-' . $marker,
        ],
      ],
    ];

    $elements['citeIDLabel'] = [
      '#title' => $this->t('Label'),
      '#type' => 'textfield',
      '#size' => 10,
      '#default_value' => $this->getSetting('citeIDLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-padding-left-15',
        ],
      ],
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    $elements['citeIDLabelIconOrLink'] = [
      '#title' => $this->t('Show as'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('citeIDLabelIconOrLink'),
      '#weight' => $weight++,
      '#options' => $this->getIdLabelOptions(),
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-padding-radios-15',
        ],
      ],
      '#attributes' => [
        'class' => [
          'citeIDLabelIconOrLink-' . $marker,
        ],
      ],
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    $elements['showIDUrlLink'] = [
      '#title' => $this->t('Link to the item'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('showIDUrlLink'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-padding-left-15',
        ],
      ],
      '#attributes' => [
        'class' => [
          'showIDUrlLink-' . $marker,
        ],
      ],
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
      ],
    ];

    $elements['citeIDOpenLinkIn'] = [
      '#title' => $this->t('Use link to'),
      '#type' => 'select',
      '#options' => $this->getOpenLinkInValues(),
      '#default_value' => $this->getSetting('citeIDOpenLinkIn'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-orcid-link-open-link-in',
        ],
      ],
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
        'disabled' => [
          '.showIDUrlLink-' . $marker => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $elements['citeIDurlnoreferrer'] = [
      '#title' => $this->t('Do not pass the current site as the referrer ("noreferrer")'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('citeIDurlnoreferrer'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-orcid-link-noreferrer',
        ],
      ],
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
        'disabled' => [
          '.showIDUrlLink-' . $marker => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $elements['citeIDurlnoopener'] = [
      '#title' => $this->t('Do not share the current page context ("noopener")'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('citeIDurlnoopener'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-orcid-link-noopener',
        ],
      ],
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
        'disabled' => [
          '.showIDUrlLink-' . $marker => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    $elements['citeIDurlnofollow'] = [
      '#title' => $this->t('Do not treat the link as an endorsement ("nofollow")'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('citeIDurlnofollow'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-orcid-link-nofollow',
        ],
      ],
      '#states' => [
        'visible' => [
          '.showIDUrl-' . $marker => [
            'checked' => TRUE,
          ],
        ],
        'disabled' => [
          '.showIDUrlLink-' . $marker => [
            'checked' => FALSE,
          ],
        ],
      ],
    ];

    return $elements;
  }

  /**
   * Sanitize settings to make sure that they are safe and valid.
   */
  protected function sanitizeSettings() {
    // Get current settings.
    $citeID_open_link_in = $this->getSetting('citeIDOpenLinkIn');
    $showIDUrl = $this->getSetting('showIDUrl');
    $showIDUrlLink = $this->getSetting('showIDUrlLink');
    $citeIDurlnoreferrer = $this->getSetting('citeIDurlnoreferrer');
    $citeIDurlnoopener = $this->getSetting('citeIDurlnoopener');
    $citeIDurlnofollow = $this->getSetting('citeIDurlnofollow');

    $citeIDLabelIconOrLink = $this->getSetting('citeIDLabelIconOrLink');

    // Get setting defaults.
    $defaults = $this->defaultSettings();

    $authorOpenLinkInValues = $this->getOpenLinkInValues();
    if (empty($citeID_open_link_in) === TRUE || isset($authorOpenLinkInValues[$citeID_open_link_in]) === FALSE) {
      $this->setSetting('citeIDOpenLinkIn', $defaults['citeIDOpenLinkIn']);
    }

    $orcidLabelOptions = $this->getIdLabelOptions();
    if (empty($citeIDLabelIconOrLink) === TRUE || isset($orcidLabelOptions[$citeIDLabelIconOrLink]) === FALSE) {
      $this->setSetting('citeIDLabelIconOrLink', $defaults['citeIDLabelIconOrLink']);
    }

    // Insure boolean values are boolean for Citation ID URL.
    $this->setSetting('showIDUrl', boolval($showIDUrl));
    $this->setSetting('showIDUrlLink', boolval($showIDUrlLink));
    $this->setSetting('citeIDurlnoreferrer', boolval($citeIDurlnoreferrer));
    $this->setSetting('citeIDurlnoopener', boolval($citeIDurlnoopener));
    $this->setSetting('citeIDurlnofollow', boolval($citeIDurlnofollow));

    if ($showIDUrl === FALSE) {
      $this->setSetting('showIDUrlLink', FALSE);
    }
  }

}
