<?php

namespace Drupal\citeref_field\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\NestedArray;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'citeref_field_widget' widget.
 *
 * A widget citation or reference renders the citeref fields inside forms.
 *
 * @FieldWidget(
 *   id = "citeref_field_widget",
 *   label = @Translation("Citeref Field Widget"),
 *   field_types = {"citeref_field"},
 * )
 */
class CiterefFieldWidget extends WidgetBase {
  /**
   * A http client.
   */
  protected ClientInterface $httpClient;

  /**
   * The configuration factory service.
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a new CustomWidget object.
   *
   * The plugin ID for the widget.
   */
  public function __construct($plugin_id,
  $plugin_definition,
        FieldDefinitionInterface $field_definition,
        array $settings,
        array $third_party_settings,
        ConfigFactoryInterface $config_factory,
        ClientInterface $http_client) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
          $plugin_id,
          $plugin_definition,
          $configuration['field_definition'],
          $configuration['settings'],
          $configuration['third_party_settings'],
          $container->get('config.factory'),
          $container->get('http_client'),
      );
  }

  /**
   * As the default setting, show all citation or reference fields.
   */
  public static function defaultSettings() {
    return array_merge(
          [
            'hideCiterefType' => FALSE,
            'hideCiterefStyle' => FALSE,
            'hideCiterefID' => FALSE,
            'hideCiterefRecord' => FALSE,
            'citeTypeLabel' => 'Citation Type',
            'citeStyleLabel' => 'Citation Style',
            'citeRecordLabel' => 'Citation',
            'citeRecordDescriptionLabel' => 'Enter citation text.',
            'citeDetailLabel' => 'Citation Details',
          ],
          parent::defaultSettings());
  }

  /**
   * Defines the form to allow for users to change the values for the setting.
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $this->sanitizeSettings();

    $weight = 0;
    $marker = rand();

    $elements['sectionCitationTypeTitle'] = [
      '#markup' => $this->t('<div><b>Citation Type Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationTypeBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeTypeLabel'] = [
      '#title' => $this->t('Citation Type Label'),
      '#type' => 'textfield',
      '#size' => 30,
      '#default_value' => $this->getSetting('citeTypeLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['sectionCitationStyleTitle'] = [
      '#markup' => $this->t('<div><b>Citation Style Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationStyleBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeStyleLabel'] = [
      '#title' => $this->t('Citation Style Label'),
      '#type' => 'textfield',
      '#size' => 30,
      '#default_value' => $this->getSetting('citeStyleLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['sectionCitationDetailTitle'] = [
      '#markup' => $this->t('<div><b>Citation Details Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationDetailBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeDetailLabel'] = [
      '#title' => $this->t('Citation Details Label'),
      '#type' => 'textfield',
      '#size' => 30,
      '#default_value' => $this->getSetting('citeDetailLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['sectionCitationRecordTitle'] = [
      '#markup' => $this->t('<div><b>Citation Text Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationRecordBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeRecordLabel'] = [
      '#title' => $this->t('Citation Text Label'),
      '#type' => 'textfield',
      '#size' => 30,
      '#default_value' => $this->getSetting('citeRecordLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['sectionCitationRecordDescription'] = [
      '#markup' => $this->t('<div><b>Citation Description Text Settings</b></div>'),
      '#weight' => $weight++,
    ];
    $elements['sectionCitationRecordDescriptionBreak'] = [
      '#markup' => '<hr>',
      '#weight' => $weight++,
    ];

    $elements['citeRecordDescriptionLabel'] = [
      '#title' => $this->t('Citation Description Text'),
      '#type' => 'textfield',
      '#size' => 30,
      '#default_value' => $this->getSetting('citeRecordDescriptionLabel'),
      '#weight' => $weight++,
      '#attributes' => [
        'autocomplete' => 'off',
        'autocapitalize' => 'none',
        'spellcheck' => 'false',
        'autocorrect' => 'off',
      ],
      '#wrapper_attributes' => [
        'class' => [
          '',
        ],
      ],
    ];

    $elements['hideCiterefType'] = [
      '#title' => $this->t('Hide Citation Type field'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('hideCiterefType'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-padding-left-15',
        ],
      ],
      '#attributes' => [
        'class' => [
          'hideCiterefType-' . $marker,
        ],
      ],
    ];

    $elements['hideCiterefStyle'] = [
      '#title' => $this->t('Hide Citation Style Language field'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('hideCiterefStyle'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-padding-left-15',
        ],
      ],
      '#attributes' => [
        'class' => [
          'hideCiterefStyle-' . $marker,
        ],
      ],
    ];

    $elements['hideCiterefID'] = [
      '#title' => $this->t('Hide Citation ID field'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('hideCiterefID'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-padding-left-15',
        ],
      ],
      '#attributes' => [
        'class' => [
          'hideCiterefID-' . $marker,
        ],
      ],
    ];

    $elements['hideCiterefRecord'] = [
      '#title' => $this->t('Hide Citation text field'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('hideCiterefRecord'),
      '#weight' => $weight++,
      '#wrapper_attributes' => [
        'class' => [
          'formatter_suite-padding-left-15',
        ],
      ],
      '#attributes' => [
        'class' => [
          'hideCiterefRecord-' . $marker,
        ],
      ],
    ];

    return $elements;
  }

  /**
   * Summarize the currently selected settings in the summary for the widget.
   */
  public function settingsSummary() {
    $summary = [];

    if ($this->getSetting('hideCiterefType')) {
      $summary[] = 'Hidden: Citation Type field ';
    }

    if ($this->getSetting('hideCiterefStyle')) {
      $summary[] = 'Hidden: Citation Style Language field ';
    }

    if ($this->getSetting('hideCiterefID')) {
      $summary[] = 'Hidden: Citation ID field ';
    }

    if ($this->getSetting('hideCiterefRecord')) {
      $summary[] = 'Hidden: Citation text field ';
    }

    if (empty($summary) === TRUE) {
      $summary[] = 'Hidden: None';
    }

    if ($this->getSetting('citeTypeLabel')) {
      $summary[] = $this->t('Labels: @citeTypeLabel, @citeStyleLabel, @citeRecordLabel, @citeRecordDescriptionLabel, @citeDetailLabel',
        [
          '@citeTypeLabel' => $this->getSetting('citeTypeLabel'),
          '@citeStyleLabel' => $this->getSetting('citeStyleLabel'),
          '@citeRecordLabel' => $this->getSetting('citeRecordLabel'),
          '@citeRecordDescriptionLabel' => $this->getSetting('citeRecordDescriptionLabel'),
          '@citeDetailLabel' => $this->getSetting('citeDetailLabel'),
        ]);
    }

    return $summary;
  }

  /**
   * Sanitize settings to make sure that they are safe and valid.
   */
  protected function sanitizeSettings() {
    $this->setSetting('hideCiterefType', boolval($this->getSetting('hideCiterefType')));
    $this->setSetting('hideCiterefStyle', boolval($this->getSetting('hideCiterefStyle')));
    $this->setSetting('hideCiterefID', boolval($this->getSetting('hideCiterefID')));
    $this->setSetting('hideCiterefRecord', boolval($this->getSetting('hideCiterefRecord')));
  }

  /**
   * Reference: https://www.drupal.org/project/drupal/issues/3143002.
   *
   * Drupal message is displayed inside each citation type on the page.
   */
  public function addMessage($message_type, $message) {
    $render_message = '<div data-drupal-messages="" class="messages-list">';
    if ($message_type === 'status') {
      $render_message .= '<div role="contentinfo" aria-labelledby="message-status-title" ' .
              'class="messages-list__item messages messages--status">' .
              '<div class="messages__header">' .
                '<h2 id="message-status-title" class="messages__title">Status message</h2>' .
              '</div><div class="messages__content">' . $message . '</div></div>';
    }
    else {
      $index = rand(100000000000000, 999999999999999);
      $render_message .= '<div class="messages messages--' . $message_type . '" role="alert" ' .
              'aria-labelledby="' . $message_type . '-' . $index . '-title" ' .
              'data-drupal-message-id="' . $message_type . '-' . $index . '" ' .
              'data-drupal-message-type="' . $message_type . '">' .
              '<div class="messages__header"><h2 id="' . $message_type . '-' . $index . '-title" class="messages__title">' .
                ucfirst($message_type) . ' message</h2></div>' .
              '<div class="messages__content">' . $message . '</div></div>';
    }
    $render_message .= '</div>';

    return $render_message;
  }

  /**
   * Get the title name in html text using a regular expression match.
   */
  public function getTitle($data): ?string {
    return preg_match('/<title[^>]*>(.*?)<\/title>/ims', $data, $matches) ? trim($matches[1]) : NULL;
  }

  /**
   * Ajax callback method gets executed after getting the citation identifier.
   *
   * And accesses the citation type APIs to get the formatted citation, or
   * validate the citation identifier in the form elements.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function citerefCallback(array &$form, FormStateInterface $form_state): mixed {
    $triggering = $form_state->getTriggeringElement();
    $value = trim($form_state->getValue($triggering['#parents']));
    $citeref_id = urlencode($value);

    $target_form_element = NestedArray::getValue($form, array_slice($triggering['#array_parents'], 0, -1));
    $result = '';
    $message = '';

    $citation_type = $target_form_element['citeref_type']['#value'];
    $citation_id_description = $this->getCitationIdDescription($citation_type);
    $citation_id_title = $this->getCitationIdTitle($citation_type);
    $target_form_element['citeref_id']['#description'] = $citation_id_description;
    $target_form_element['citeref_id']['#title'] = $citation_id_title;

    if (!empty($value)) {
      // Process the "DOI" as the citation type.
      if ($citation_type === 'DOI') {
        // Regular expressions to validate or scrape for modern Crossref DOIs.
        $pattern = '/^10[.]/';
        if (!preg_match($pattern, $value)) {
          $messages = $this->t('The given DOI, @value is not valid.', ['@value' => $value]);
          $message = $this->addMessage('error', $messages);
        }
        else {
          $config = $this->configFactory->getEditable('citeref_field.settings');
          try {
            $citation_style = $target_form_element['citeref_style']['#value'];
            $doi_url = $config->get('doi_url') . '/' . $citeref_id;
            $request = $this->httpClient->request('GET', $doi_url,
                  ['headers' => ['Accept' => 'text/x-bibliography; style=' . $citation_style]]);

            $status = $request->getStatusCode();
            if ($status != 200) {
              $messages = $this->t('The DOI, @value is not found.', ['@value' => $value]);
              $message = $this->addMessage('warning', $messages);
            }
            else {
              $result = $request->getBody()->getContents();
              $html_string = '<head>';
              if (strpos($result, $html_string)) {
                $messages = $this->t('The DOI, @value is not available as the citation style.', ['@value' => $value]);
                $message = $this->addMessage('status', $messages);
                $result = NULL;
              }
              else {
                $filter_identifiers = ['1.', '(1)', '[1]'];
                foreach ($filter_identifiers as $filter_identifier) {
                  if (str_starts_with($result, $filter_identifier)) {
                    $result = substr($result, strlen($filter_identifier));
                    break;
                  }
                }
              }
            }
          }
          catch (\Exception $e) {
            // Wrap the markup in a div.
            $messages = $this->t('The DOI, @value is not found.', ['@value' => $value]);
            $message = $this->addMessage('warning', $messages);
            $result = NULL;
          }
        }
        // Process the "Handle" as the citation type.
      }
      elseif ($citation_type === 'Handle') {
        $config = $this->configFactory->getEditable('citeref_field.settings');
        try {
          $params = '?type=URL&type=EMAIL';
          $request_url = $config->get('handle_net_url') . '/' . $citeref_id . $params;
          $request = $this->httpClient->request('GET', $request_url);

          $status = $request->getStatusCode();
          if ($status != 200) {
            $messages = $this->t('The Handle, @value is not found.', ['@value' => $value]);
            $message = $this->addMessage('warning', $messages);
          }
          else {
            $json_result = (string) $request->getBody();
            $json = json_decode($json_result, TRUE);
            if (!empty($json['values'])) {
              $url_list = [];
              $emali_list = [];
              foreach ($json['values'] as $hdl_value) {
                if ($hdl_value['type'] === 'URL') {
                  $url_list[] = $hdl_value['data']['value'];
                }
                elseif ($hdl_value['type'] === 'EMAIL') {
                  $emali_list[] = $hdl_value['data']['value'];
                }
              }

              if (!empty($url_list)) {
                $result .= $this->t('Referred URLs: @url_list', ['@url_list' => implode(PHP_EOL, $url_list)]);
              }
              if (!empty($emali_list)) {
                $result .= $this->t('@blankEmail addresses: @email_list',
                  [
                    '$blank' => ' ',
                    '@email_list' => implode(PHP_EOL, $emali_list),
                  ]);
              }
            }
            // Wrap the markup in a div.
            $messages = $this->t('The Handle, @value is valid.', ['@value' => $value]);
            $message = $this->addMessage('status', $messages);
          }
        }
        catch (\Exception $e) {
          // Wrap the markup in a div.
          $messages = $this->t('The Handle, @value is not found.', ['@value' => $value]);
          $message = $this->addMessage('warning', $messages);
          $result = NULL;
        }
        // Process the "ARK" as the citation type.
      }
      elseif ($citation_type === 'ARK') {
        $config = $this->configFactory->getEditable('citeref_field.settings');
        try {
          $params = '/??';
          // The ARK resolver.
          $request_url = $config->get('ark_url') . $citeref_id . $params;
          $request = $this->httpClient->request('GET', $request_url);

          $status = $request->getStatusCode();
          if ($status != 200) {
            $messages = $this->t('The ARK, @value is not found.', ['@value' => $value]);
            $message = $this->addMessage('warning', $messages);
          }
          else {
            $messages = $this->t('The ARK, @value is valid.', ['@value' => $value]);
            $message = $this->addMessage('status', $messages);

            $result = $request->getBody()->getContents();
            $html_string = '<head>';
            if (strpos($result, $html_string)) {
              $result = NULL;
            }
          }
        }
        catch (\Exception $e) {
          // Wrap the markup in a div.
          $messages = $this->t('The ARK, @value is not found.', ['@value' => $value]);
          $message = $this->addMessage('warning', $messages);
          $result = NULL;
        }
        // Process the "URL" as the citation type.
      }
      elseif ($citation_type === 'URL') {
        if (!UrlHelper::isValid($value, TRUE)) {
          $messages = $this->t('The syntax of the given URL, @value is not valid.', ['@value' => $value]);
          $message = $this->addMessage('error', $messages);
        }
        else {
          try {
            $request = $this->httpClient->request('GET', $value);
            $status = $request->getStatusCode();
            if ($status != 200) {
              $messages = $this->t('The URL, @value is not found.', ['@value' => $value]);
              $message = $this->addMessage('error', $messages);
            }
            else {
              $messages = $this->t('The URL, @value is valid', ['@value' => $value]);
              $message = $this->addMessage('status', $messages);

              $html = $request->getBody()->getContents();
              $title = $this->getTitle($html);
              date_default_timezone_set('America/Los_Angeles');
              $now = new \DateTime('now');
              $day = $now->format('d');
              $month = $now->format('M');
              $year = $now->format('Y');
              // URL: Title. Accessed Date from URL.
              $result = 'URL: ' . $title . '. ' . 'Accessed ' . $day . ' ' . $month . ' ' . $year . ' from ' . $value;
            }
          }
          catch (\Exception $e) {
            // Wrap the markup in a div.
            $messages = $this->t('The URL, @value is not found.', ['@value' => $value]);
            $message = $this->addMessage('error', $messages);
            $result = NULL;
          }
        }
        // Process the "URN ISSN/ISBN" as the citation type.
      }
      elseif ($citation_type === 'URN') {
        $value = trim(strtolower($value));
        if (str_starts_with($value, 'isbn:')) {
          $value = str_replace('-', '', $value);
          $isbn_number = substr($value, strlen('isbn:'));
          [$message, $result] = $this->getIsbn($isbn_number);
        }
        elseif (str_starts_with($value, 'issn:')) {
          $issn_number = substr($value, strlen('issn:'));
          [$message, $result] = $this->getIssn($issn_number);
        }
        else {
          $result = NULL;
        }
      }
      else {
        return $target_form_element;
      }
    }

    $target_form_element['message']['#markup'] = $message;
    $target_form_element['citeref_record']['#value'] = $result;
    $target_form_element['_weight'] = [];

    return $target_form_element;
  }

  /**
   * Ajax callback method gets executed after getting the citation style.
   *
   * Accesses the citation type APIs to get the formatted citation
   * in the form elements.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function citerefAutocompleteCallback(array &$form, FormStateInterface $form_state): mixed {
    $triggering = $form_state->getTriggeringElement();
    $citation_style = trim($form_state->getValue($triggering['#parents']));

    $target_form_element = NestedArray::getValue($form, array_slice($triggering['#array_parents'], 0, -1));
    $result = '';
    $message = '';

    $value = $target_form_element['citeref_id']['#value'];
    if ($value !== NULL) {
      $value = trim($value);
    }
    $citeref_id = urlencode($value);

    if (!empty($value)) {
      // Process the "DOI" as the citation type.
      // Regular expressions to validate or scrape for modern Crossref DOIs.
      $pattern = '/^10[.]/';
      if (!preg_match($pattern, $value)) {
        $messages = $this->t('The given DOI, @value is not valid.', ['@value' => $value]);
        $message = $this->addMessage('error', $messages);
      }
      else {
        $config = $this->configFactory->getEditable('citeref_field.settings');
        try {
          // $citation_style = $target_form_element['citeref_style']['#value'];
          $doi_url = $config->get('doi_url') . '/' . $citeref_id;
          $request = $this->httpClient->request('GET', $doi_url,
            ['headers' => ['Accept' => 'text/x-bibliography; style=' . $citation_style]]);

          $status = $request->getStatusCode();
          if ($status != 200) {
            $messages = $this->t('The DOI, @value is not found.', ['@value' => $value]);
            $message = $this->addMessage('warning', $messages);
          }
          else {
            $result = $request->getBody()->getContents();
            $html_string = '<head>';
            if (strpos($result, $html_string)) {
              $messages = $this->t('The DOI, @value is not available as the citation style.', ['@value' => $value]);
              $message = $this->addMessage('status', $messages);
              $result = NULL;
            }
            else {
              $filter_identifiers = ['1.', '(1)', '[1]'];
              foreach ($filter_identifiers as $filter_identifier) {
                if (str_starts_with($result, $filter_identifier)) {
                  $result = substr($result, strlen($filter_identifier));
                  break;
                }
              }
            }
          }
        }
        catch (\Exception $e) {
          // Wrap the markup in a div.
          $messages = $this->t('The DOI, @value is not found.', ['@value' => $value]);
          $message = $this->addMessage('warning', $messages);
          $result = NULL;
        }
      }
    }
    else {
      $messages = $this->t('Please enter DOI.');
      $message = $this->addMessage('error', $messages);
    }

    $target_form_element['message']['#markup'] = $message;
    $target_form_element['citeref_record']['#value'] = $result;
    $target_form_element['_weight'] = [];

    return $target_form_element;
  }

  /**
   * This function is used to determine whether ar not ISBN is.
   *
   * Parameter Description:
   */
  public function isIsbn($isbn) {
    $len = strlen($isbn);

    // Get the ISBN, various formats are supported. e.g. 0073545775 (10-digit)
    // or 9780073545776 (13-digit)
    if ($len != 10 && $len != 13) {
      return 0;
    }
  }

  /**
   * Retrieves the book information from google book api using isbn number.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function getIsbn($isbn): array {
    if ($this->isIsbn($isbn)) {
      $messages = $this->t('The ISBN, @isbn is not valid.', ['@isbn' => $isbn]);
      $message = $this->addMessage('warning', $messages);
      $result = NULL;
    }
    else {
      $config = $this->configFactory->getEditable('citeref_field.settings');
      try {
        $params = '?q=isbn:' . $isbn;
        $request_url = $config->get('isbn_url') . $params;
        $request = $this->httpClient->request('GET', $request_url);
        $status = $request->getStatusCode();
        if ($status != 200) {
          $messages = $this->t('The ISBN, @isbn is not found.', ['@isbn' => $isbn]);
          $message = $this->addMessage('warning', $messages);
          $result = NULL;
        }
        else {
          $json_result = (string) $request->getBody();
          $json = json_decode($json_result, TRUE);
          if (isset($json['items'][0])) {
            $volume_info = $json['items'][0];
            $result = '';
            // Displays title, author, publisher, publication date.
            if (isset($json['items'][0]['volumeInfo']['authors'])) {
              $authors = $json['items'][0]['volumeInfo']['authors'];
              $authors = implode(',', $authors);
              $result .= $authors . '. ';
            }
            if (isset($volume_info['volumeInfo']['title'])) {
              $title = $volume_info['volumeInfo']['title'];
              $result .= $title . '. ';
            }
            if (isset($volume_info['volumeInfo']['publisher'])) {
              $publisher = $volume_info['volumeInfo']['publisher'];
              $result .= $publisher . ', ';
            }
            if (isset($volume_info['volumeInfo']['publishedDate'])) {
              $publication_date = $volume_info['volumeInfo']['publishedDate'] ?? NULL;
              $result .= $publication_date . '.';
            }
            // Wrap the markup in a div.
            $messages = $this->t('The ISBN, @isbn is valid.', ['@isbn' => $isbn]);
            $message = $this->addMessage('status', $messages);
          }
          else {
            $messages = $this->t('The ISBN, @isbn is not found.', ['@isbn' => $isbn]);
            $message = $this->addMessage('warning', $messages);
            $result = NULL;
          }
        }
      }
      catch (\Exception $e) {
        // Wrap the markup in a div.
        $messages = $this->t('The ISBN, @isbn is not found.', ['@isbn' => $isbn]);
        $message = $this->addMessage('warning', $messages);
        $result = NULL;
      }
    }

    return [$message, $result];
  }

  /**
   * The function validates the issn number.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function getIssn($issn): array {
    $config = $this->configFactory->getEditable('citeref_field.settings');
    try {
      $request_url = $config->get('issn_url') . '/' . $issn;
      $request = $this->httpClient->request('GET', $request_url);
      $status = $request->getStatusCode();
      if ($status != 200) {
        $messages = $this->t('The ISSN, @issn is not found.', ['@issn' => $issn]);
        $message = $this->addMessage('warning', $messages);
      }
      else {
        $contents = $request->getBody()->getContents();

        // Wrap the markup in a div.
        $issn_needle = 'Welcome to The ISSN Portal';
        $issn_pos = strpos($contents, $issn_needle);
        if ($issn_pos === FALSE) {
          $messages = $this->t('The ISSN, @issn is valid.', ['@issn' => $issn]);
          $message = $this->addMessage('status', $messages);
        }
        else {
          $messages = $this->t('The ISSN, @issn is not found.', ['@issn' => $issn]);
          $message = $this->addMessage('warning', $messages);
        }
      }
      $result = NULL;
    }
    catch (\Exception $e) {
      // Wrap the markup in a div.
      $messages = $this->t('The ISSN, @issn is not found.', ['@issn' => $issn]);
      $message = $this->addMessage('warning', $messages);
      $result = NULL;
    }

    return [$message, $result];
  }

  /**
   * The function displays the description of the citation identifier.
   */
  public function getCitationIdDescription($citation_type): string {
    return match ($citation_type) {
      'DOI' => 'Example: 10.1126/science.169.3946.635',
            'Handle' => 'Example: 20.1000/100',
            'ARK' => 'Example: 67531/metadc2190444',
            'URL' => 'Example: https://sciencegateways.org',
            'URN' => '<div>Examples: <ul><li>isbn:0-345-33971-1</li><li>issn:0028-0836</li><li>ietf:rfc:2141</li><li>0015-6914(19960101)157:1%3C62:KTSW%3E2.0.TX;2-F</li></ul></div>',
            'Other' => 'Any available unique identifier'
    };
  }

  /**
   * The function displays the title of the citation identifier.
   */
  public function getCitationIdTitle($citation_type): string {
    return match ($citation_type) {
      'DOI' => 'Enter DOI',
            'Handle' => 'Enter Handle',
            'ARK' => 'Enter ARK',
            'URL' => 'Enter URL',
            'URN' => 'Enter URN',
            'Other' => 'Enter other'
    };
  }

  /**
   * Show/hide the citation style field, based on the citation type.
   */
  public function citerefSelectCallback(array &$form, FormStateInterface $form_state): mixed {
    $triggering = $form_state->getTriggeringElement();
    $citeref_type = trim($form_state->getValue($triggering['#parents']));

    $target_form_element = NestedArray::getValue($form, array_slice($triggering['#array_parents'], 0, -1));

    if (!empty($citeref_type)) {
      $citation_id_title = $this->getCitationIdTitle($citeref_type);
      $citation_id_description = $this->getCitationIdDescription($citeref_type);
      $target_form_element['citeref_id']['#title'] = $citation_id_title;
      $target_form_element['citeref_id']['#description'] = $citation_id_description;
      $target_form_element['citeref_style']['#value'] = 'apa';
      $target_form_element['citeref_id']['#value'] = NULL;
      $target_form_element['citeref_record']['#value'] = NULL;
      $target_form_element['_weight'] = [];
    }

    return $target_form_element;
  }

  /**
   * Represent the citation ot reference field widget.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $field_name = $this->fieldDefinition->getName();
    $index = Html::getUniqueId('osp-citeref');

    $element['osp_citeref_field'] = [
      '#type' => 'details',
      '#title' => $this->t('@citation_detail',
              ['@citation_detail' => $this->getSetting('citeDetailLabel')]),
      '#open' => TRUE,
      '#attributes' => [
        'id' => "citeref-data-wrapper-$index",
      ],
      '#prefix' => '<div id="citeref-data-wrapper-' . $index . '">',
      '#suffix' => '</div>',
    ];

    $element['osp_citeref_field']['message'] = [
      '#type' => 'markup',
      '#markup' => '',
    ];
    $citation_type = $items[$delta]->citeref_type ?? 'DOI';

    if (!$this->getSetting('hideCiterefType')) {
      $element['osp_citeref_field']['citeref_type'] = [
        '#type' => 'select',
        '#title' => $this->t('@citation_type',
                ['@citation_type' => $this->getSetting('citeTypeLabel')]),
        '#description' => $this->t('Select identifier type.'),
        '#options' => [
          'DOI' => $this->t('DOI'),
          'Handle' => $this->t('Handle'),
          'ARK' => $this->t('ARK'),
          'URL' => $this->t('URL'),
          'URN' => $this->t('URN'),
          'Other' => $this->t('Other'),
        ],
        '#default_value' => $citation_type,
        '#ajax' => [
          'callback' => [$this, 'citerefSelectCallback'],
          'wrapper' => "citeref-data-wrapper-$index",
            // Or TRUE to prevent re-focusing on the triggering element.
          'disable-refocus' => FALSE,
          'event' => 'change',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Verifying entry...'),
          ],
        ],
      ];
    }

    if (!$this->getSetting('hideCiterefStyle')) {
      $element['osp_citeref_field']['citeref_style'] = [
        '#type' => 'textfield',
        '#title' => $this->t('@citation_type',
                ['@citation_type' => $this->getSetting('citeStyleLabel')]),
        '#autocomplete_route_name' => 'citeref_field.autocomplete',
        '#description' => $this->t('Select Formatting Style'),
        '#default_value' => $items[$delta]->citeref_style ?? 'apa',
        '#ajax' => [
          'callback' => [$this, 'citerefAutocompleteCallback'],
          'event' => 'autocompleteclose',
          'wrapper' => "citeref-data-wrapper-$index",
          'method' => 'replace',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Updating...'),
          ],
        ],
        '#attributes' => [
          'data-disable-refocus' => 'true',
        ],
      ];

      $element['osp_citeref_field']['citeref_style']['#states'] = [
        'visible' => [
                [
                    [':input[name="' . $field_name . '[' . $delta . '][osp_citeref_field][citeref_type]"]' => ['value' => 'DOI']],
                ],
        ],
      ];
    }

    if (!$this->getSetting('hideCiterefID')) {
      $citeref_id_description = $this->getCitationIdDescription($citation_type);
      $citeref_id_title = $this->getCitationIdTitle($citation_type);
      $safeDescription = Markup::create($citeref_id_description);

      $element['osp_citeref_field']['citeref_id'] = [
        '#type' => 'textfield',
        '#title' => $this->t('@citeref_id_title', ['@citeref_id_title' => $citeref_id_title]),
        '#maxlength' => 255,
        '#default_value' => $items[$delta]->citeref_id ?? NULL,
        '#description' => $this->t('@citeref_id_description', ['@citeref_id_description' => $safeDescription]),
        '#attributes' => [
          // Custom class for identifying the text field.
          'class' => ['debounce-ajax'],
          // Store wrapper ID in data attribute.
          'data-wrapper' => "citeref-data-wrapper-$index",
          'data-disable-refocus' => 'true',
        ],
      ];

      $element['osp_citeref_field']['citeref_id']['#ajax'] = [
        'callback' => [$this, 'citerefCallback'],
        // Listen for the custom debouncedInput event.
        'event' => 'debouncedInput',
        'wrapper' => "citeref-data-wrapper-$index",
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Updating...'),
        ],
      ];

    }

    if (!$this->getSetting('hideCiterefRecord')) {
      $element['osp_citeref_field']['citeref_record'] = [
        '#title' => $this->t('@citation',
                ['@citation' => $this->getSetting('citeRecordLabel')]),
        '#type' => 'textarea',
        '#default_value' => $items[$delta]->citeref_record ?? NULL,
        '#description' => $this->t('@citation_desc',
                ['@citation_desc' => $this->getSetting('citeRecordDescriptionLabel')]),
      ];
    }

    $element['#attached']['library'][] = 'citeref_field/citeref_field_twig_default';
    $element['#attached']['library'][] = 'citeref_field/citeref_field_ui_default';

    // If cardinality is 1, ensure a proper label is output for the field.
    // Wrap everything in a details element.
    if ($this->fieldDefinition->getFieldStorageDefinition()->getCardinality() == 1) {
      $element += [
        '#type' => 'markup',
        '#markup' => $this->t('<h4 style="margin-bottom: 1em;">@Author</h4><hr style="color: #0a6eb4">',
                ['@Author' => ucfirst($field_name)]),
      ];
    }

    return $element;
  }

  /**
   * Massages the form values into the format expected for field values.
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $delta => $value) {
      if (isset($value['osp_citeref_field']['citeref_type'])) {
        if ($value['osp_citeref_field']['citeref_type'] === '') {
          $values[$delta]['citeref_type'] = NULL;
        }
        else {
          $values[$delta]['citeref_type'] = $value['osp_citeref_field']['citeref_type'];
        }
      }
      else {
        $values[$delta]['citeref_type'] = NULL;
      }

      if (isset($value['osp_citeref_field']['citeref_style'])) {
        if ($value['osp_citeref_field']['citeref_style'] === '') {
          $values[$delta]['citeref_style'] = 'apa';
        }
        else {
          $values[$delta]['citeref_style'] = $value['osp_citeref_field']['citeref_style'];
        }
      }
      else {
        $values[$delta]['citeref_style'] = 'apa';
      }

      if (isset($value['osp_citeref_field']['citeref_id'])) {
        if ($value['osp_citeref_field']['citeref_id'] === '') {
          $values[$delta]['citeref_id'] = NULL;
        }
        else {
          $values[$delta]['citeref_id'] = $value['osp_citeref_field']['citeref_id'];
        }
      }
      else {
        $values[$delta]['citeref_id'] = NULL;
      }

      if (isset($value['osp_citeref_field']['citeref_record'])) {
        if ($value['osp_citeref_field']['citeref_record'] === '') {
          $values[$delta]['citeref_record'] = NULL;
        }
        else {
          $values[$delta]['citeref_record'] = $value['osp_citeref_field']['citeref_record'];
        }
      }
      else {
        $values[$delta]['citeref_record'] = NULL;
      }
    }

    return $values;
  }

}
